@@ -1,19 +1,25 @@
-import datetime
 import glob
-from typing import Any, Dict, Iterator, List, Optional, Sequence, Type, Union, Tuple
+from typing import Iterator, List, Type, Union, Tuple
 
 import albumentations as A
 import numpy as np
-import torch.nn
 import torch.utils.data as thd
 import webdataset as wds
 from torch.utils.data.dataset import T_co
 import logging
 import pickle
-import io
 import random
 import os
-
+from satflow.data.utils.normalization import metnet_normalization, standard_normalization
+from satflow.data.utils.utils import (
+    create_time_layer,
+    load_np,
+    binarize_mask,
+    create_pixel_coord_layers,
+    check_channels,
+    crop_center,
+    load_config,
+)
 
 logger = logging.getLogger("satflow.dataset")
 logger.setLevel(logging.WARN)
@@ -35,90 +41,6 @@ def get_dataset(name: str) -> Type[thd.IterableDataset]:
     return REGISTERED_DATASET_CLASSES[name]
 
 
-def create_time_layer(dt: datetime.datetime, shape):
-    """Create 3 layer for current time of observation"""
-    month = dt.month / 12
-    day = dt.day / 31
-    hour = dt.hour / 24
-    # minute = dt.minute / 60
-    return np.stack([np.full(shape, month), np.full(shape, day), np.full(shape, hour)], axis=-1)
-
-
-def load_np(data):
-    import numpy.lib.format
-
-    stream = io.BytesIO(data)
-    return numpy.lib.format.read_array(stream)
-
-
-def space_to_depth(x: np.ndarray, downsample: int) -> np.ndarray:
-    batch, height, width, depth = x.shape
-    reduced_height = height // downsample
-    reduced_width = width // downsample
-    y = x.reshape((batch, reduced_height, downsample,
-                  reduced_width, downsample, depth))
-    z = np.swapaxes(y, 2, 3).reshape((batch, reduced_height, reduced_width, -1))
-    return z
-
-def asStride(arr,sub_shape,stride):
-    '''Get a strided sub-matrices view of an ndarray.
-    See also skimage.util.shape.view_as_windows()
-    '''
-    s0,s1=arr.strides[:2]
-    m1,n1=arr.shape[:2]
-    m2,n2=sub_shape
-    view_shape=(1+(m1-m2)//stride[0],1+(n1-n2)//stride[1],m2,n2)+arr.shape[2:]
-    strides=(stride[0]*s0,stride[1]*s1,s0,s1)+arr.strides[2:]
-    subs=np.lib.stride_tricks.as_strided(arr,view_shape,strides=strides)
-    return subs
-
-def poolingOverlap(mat,ksize,stride=None,method='max',pad=False):
-    '''Overlapping pooling on 2D or 3D data.
-    <mat>: ndarray, input array to pool.
-    <ksize>: tuple of 2, kernel size in (ky, kx).
-    <stride>: tuple of 2 or None, stride of pooling window.
-              If None, same as <ksize> (non-overlapping pooling).
-    <method>: str, 'max for max-pooling,
-                   'mean' for mean-pooling.
-    <pad>: bool, pad <mat> or not. If no pad, output has size
-           (n-f)//s+1, n being <mat> size, f being kernel size, s stride.
-           if pad, output has size ceil(n/s).
-    Return <result>: pooled matrix.
-    '''
-
-    m, n = mat.shape[:2]
-    ky,kx=ksize
-    if stride is None:
-        stride=(ky,kx)
-    sy,sx=stride
-
-    _ceil=lambda x,y: int(np.ceil(x/float(y)))
-
-    if pad:
-        ny=_ceil(m,sy)
-        nx=_ceil(n,sx)
-        size=((ny-1)*sy+ky, (nx-1)*sx+kx) + mat.shape[2:]
-        mat_pad=np.full(size,np.nan)
-        mat_pad[:m,:n,...]=mat
-    else:
-        mat_pad=mat[:(m-ky)//sy*sy+ky, :(n-kx)//sx*sx+kx, ...]
-
-    view=asStride(mat_pad,ksize,stride)
-
-    if method=='max':
-        result=np.nanmax(view,axis=(2,3))
-    else:
-        result=np.nanmean(view,axis=(2,3))
-
-    return result
-
-def binarize_mask(mask):
-    """Binarize mask, taking max value as the data, and setting everything else to 0"""
-    tmp_mask = np.zeros_like(mask)
-    tmp_mask[np.isclose(np.round(mask), 2)] = 1
-    return tmp_mask
-
-
 # Taken from training set
 MSG_MEAN = {
     "HRV": 14.04300588,
@@ -154,75 +76,6 @@ TOPO_MEAN = 224.3065682349895
 TOPO_STD = 441.7514422990341
 
 
-def create_pixel_coord_layers(x_dim: int, y_dim: int, with_r: bool = False) -> np.ndarray:
-    """
-    Creates Coord layer for CoordConv model
-
-    :param x_dim: size of x dimension for output
-    :param y_dim: size of y dimension for output
-    :param with_r: Whether to include polar coordinates from center
-    :return: (2, x_dim, y_dim) or (3, x_dim, y_dim) array of the pixel coordinates
-    """
-    xx_ones = np.ones([1, x_dim], dtype=np.int32)
-    xx_ones = np.expand_dims(xx_ones, -1)
-
-    xx_range = np.expand_dims(np.arange(x_dim), 0)
-    xx_range = np.expand_dims(xx_range, 1)
-
-    xx_channel = np.matmul(xx_ones, xx_range)
-    xx_channel = np.expand_dims(xx_channel, -1)
-
-    yy_ones = np.ones([1, y_dim], dtype=np.int32)
-    yy_ones = np.expand_dims(yy_ones, 1)
-
-    yy_range = np.expand_dims(np.arange(y_dim), 0)
-    yy_range = np.expand_dims(yy_range, -1)
-
-    yy_channel = np.matmul(yy_range, yy_ones)
-    yy_channel = np.expand_dims(yy_channel, -1)
-
-    xx_channel = xx_channel.astype("float32") / (x_dim - 1)
-    yy_channel = yy_channel.astype("float32") / (y_dim - 1)
-
-    xx_channel = xx_channel * 2 - 1
-    yy_channel = yy_channel * 2 - 1
-    ret = np.stack([xx_channel, yy_channel], axis=0)
-
-    if with_r:
-        rr = np.sqrt(np.square(xx_channel - 0.5) + np.square(yy_channel - 0.5))
-        ret = np.concatenate([ret, np.expand_dims(rr, axis=0)], axis=0)
-    ret = np.moveaxis(ret, [1], [0])
-    return ret
-
-
-def check_channels(config: dict) -> int:
-    """
-    Checks the number of channels needed per timestep, to use for preallocating the numpy array
-    Is not the same as the one for training, as that includes the number of channels after the array is partly
-    flattened
-    Args:
-        config:
-
-    Returns:
-
-    """
-    channels = len(config.get("bands", []))
-    channels = channels + 1 if config.get("use_mask", False) else channels
-    channels = (
-        channels + 3
-        if config.get("use_time", False) and not config.get("time_aux", False)
-        else channels
-    )
-    # if config.get("time_as_channels", False):
-    # Calc number of channels + inital ones
-    #    channels = channels * (config["num_timesteps"] + 1)
-    channels = channels + 1 if config.get("use_topo", False) else channels
-    channels = channels + 3 if config.get("use_latlon", False) else channels
-    channels = channels + 2 if config.get("add_pixel_coords", False) else channels
-    channels = channels + 1 if config.get("add_polar_coords", False) else channels
-    return channels
-
-
 @register_dataset
 class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
     def __init__(self, datasets: List[wds.WebDataset], config: dict, train: bool = True):
@@ -278,6 +131,7 @@ class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
         self.return_target_stack = config.get("stack_targets", False)
         self.time_as_channels = config.get("time_as_channels", False)
         self.add_pixel_coords = config.get("add_pixel_coords", False)
+        self.metnet_norm = config.get("metnet_normalization", False)
 
         # Number of channels in final one
         self.num_channels = check_channels(config)
@@ -415,6 +269,8 @@ class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
             img_cube[:, :, self.num_bands : self.num_bands + 3] = t
         if self.use_mask:
             mask = binarize_mask(load_np(sample[f"cloudmask.{idx:03d}.npy"]))
+            # Resample between -1 and 1 like the rest
+            mask = (mask * 2) - 1.0
             img_cube[:, :, -1] = mask
         return img_cube, target
 
@@ -431,6 +287,9 @@ class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
                 np.random.seed(42)
                 # Have to set Python random seed for Albumentations
                 random.seed(a=42)
+                # Reduce number of repeat examples from each source, gives more variety in the inputs
+                self.num_crops = 5
+                self.num_times = 4
             for source in sources:
                 try:
                     sample = next(source)
@@ -450,7 +309,7 @@ class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
                     or self.num_timesteps * self.skip_timesteps + 1
                     >= available_steps - self.forecast_times
                 ):
-                    logger.warning(
+                    logger.debug(
                         f"Issue with {self.num_timesteps * self.skip_timesteps + 1} >= {available_steps - self.forecast_times} with {available_steps} available timesteps"
                     )
                     continue  # Skip this sample as it is missing timesteps, or has none
@@ -494,12 +353,26 @@ class SatFlowDataset(thd.IterableDataset, wds.Shorthands, wds.Composable):
                                 f"After Time Changes Image/Masks Shape: {self.input_cube.shape} {self.target_cube.shape}"
                             )
                             self.input_cube[:, : self.num_bands, :, :] = (
-                                self.input_cube[:, : self.num_bands, :, :] - self.mean
-                            ) / self.std
+                                metnet_normalization(self.input_cube[:, : self.num_bands, :, :])
+                                if self.metnet_norm
+                                else standard_normalization(
+                                    self.input_cube[:, : self.num_bands, :, :],
+                                    std=self.std,
+                                    mean=self.mean,
+                                )
+                            )
                             if self.use_image:
                                 self.target_image_cube[:, : self.num_bands, :, :] = (
-                                    self.target_image_cube[:, : self.num_bands, :, :] - self.mean
-                                ) / self.std
+                                    metnet_normalization(
+                                        self.target_image_cube[:, : self.num_bands, :, :]
+                                    )
+                                    if self.metnet_norm
+                                    else standard_normalization(
+                                        self.target_image_cube[:, : self.num_bands, :, :],
+                                        std=self.std,
+                                        mean=self.mean,
+                                    )
+                                )
                             start_channel = self.num_bands + 3 if self.use_time else self.num_bands
                             start_channel = start_channel + 1 if self.use_mask else start_channel
                             logger.info(f"Start Channel: {start_channel}")
@@ -773,14 +646,6 @@ class OpticalFlowDataset(SatFlowDataset):
                         yield prev_mask, mask, target_mask, image, prev_image
 
 
-def crop_center(img: np.ndarray, cropx: int, cropy: int) -> np.ndarray:
-    """Crops center of image through timestack, fails if all the images are concatenated as channels"""
-    t, c, y, x = img.shape
-    startx = x // 2 - (cropx // 2)
-    starty = y // 2 - (cropy // 2)
-    return img[:, :, starty : starty + cropy, startx : startx + cropx]
-
-
 class FileDataset(thd.Dataset):
     def __getitem__(self, index) -> T_co:
         try:
@@ -813,11 +678,6 @@ if __name__ == "__main__":
     # NIR1.6, VIS0.8 and VIS0.6 RGB for near normal view
     import numpy as np
     import webdataset as wds
-    import yaml
-
-    def load_config(config_file):
-        with open(config_file, "r") as cfg:
-            return yaml.load(cfg, Loader=yaml.FullLoader)["config"]
 
     dataset = wds.WebDataset("../../datasets/satflow-test.tar")
     config = load_config("../tests/configs/satflow.yaml")
